package jp.gr.java_conf.abagames.brick;

import jp.gr.java_conf.abagames.brick.shapes.*;
import jp.gr.java_conf.abagames.util.*;
import jp.gr.java_conf.abagames.util.gl4j.*;
import gl4java.*;
import java.util.Random;

/**
  block
  */
public class Block extends TrnsObjDSFix implements IMove {

  private BlockShape blockShape = new BlockShape();

  private Racket racket;
  private BallManager ballManager;
  private boolean existf;
  private GameManager gameManager;

  public Block(Racket racket, GameManager gameManager, GLFunc gl) {
    super(gl);
    shape = blockShape;
    existf = false;
    this.racket = racket;
    this.gameManager = gameManager;
  }

  public void bindBallManager(BallManager ballManager) {
    this.ballManager = ballManager;
  }

  public void clear() {
    existf = false;
  }
  
  private int type, hp;
  private float cx, cy;

  public final void set(float x, float y, int type, int hp) {
    cx = this.x = x; cy = this.y = y;
    mv = 0;
    z = -20;
    this.type = type; this.hp = hp;
    blockShape.setHeight(hp);
    existf = true;
    switch ( type ) {
      case 0:
        blockShape.setColor(0.7f, 0.5f, 1);
        break;
      case 1:
        blockShape.setColor(0.9f, 0.7f, 0.1f);
        break;
      case 2:
        blockShape.setColor(0.5f, 1, 0.7f);
        break;
      case 3:
        blockShape.setColor(0.7f, 0.9f, 1);
        break;
      case 4:
        blockShape.setColor(1, 0.5f, 0.5f);
        break;
      case 5:
        blockShape.setColor(1, 1, 0.3f);
        break;
    }
    setBlockBrightness();
  }

  private void setBlockBrightness() {
    switch (hp) {
      case 1:
        blockShape.setBrightness(0);
        break;
      case 2:
        blockShape.setBrightness(0.4f);
        break;
      case 3:
        blockShape.setBrightness(0.5f);
        break;
      case 4:
        blockShape.setBrightness(0.6f);
        break;
      case 5:
        blockShape.setBrightness(0.7f);
        break;
      default:
        blockShape.setBrightness(0.8f);
        break;
    }
  }

  private Random rnd = new Random();

  public final void hit() {
    mv = 0.3f;
    float bcx = x+BlockManager.BLKWDT/2, bcy = y-BlockManager.BLKHGT/2;
    hp--;
    if ( hp <= 0 ) {
      gameManager.breakBlock();
      existf = false;
      int deg;
      float rx = racket.x + racket.width/2;
      switch ( type ) {
        case 0:
        case 1:
          deg = DegUtil.getDeg(rx-bcx, racket.y-bcy)%360;
          ballManager.addBall(bcx, bcy, deg, 0.3f, -200);
          break;
        case 2:
          deg = DegUtil.getDeg(rx-bcx, racket.y-bcy)%360;
          ballManager.addBall(bcx, bcy, deg, 0.3f, -600);
          ballManager.addBall(bcx, bcy, deg, 0.4f, -400);
          ballManager.addBall(bcx, bcy, deg, 0.5f, -200);
          break;
        case 3:
          deg = DegUtil.getDeg(rx-bcx, racket.y-bcy)%360;
          deg -= 6; if ( deg < 0 ) deg += 360;
          for ( int i=0 ; i<7 ; i++ ) {
            ballManager.addBall(bcx, bcy, deg, 0.3f, -500);
            deg += 2; if ( deg >= 360 ) deg -= 360;
          }
          break;
        case 4:
          deg = 0;
          for ( int i=0 ; i<20 ; i++ ) {
            ballManager.addBall(bcx, bcy, deg, 0.3f, 300);
            deg += 18;
          }
          break;
        case 5:
          deg = DegUtil.getDeg(rx-bcx, racket.y-bcy)%360;
          ballManager.addBall(bcx, bcy, deg, 0.3f, -999999999);
          break;
      }
    } else {
      gameManager.hitBlock();
      blockShape.setHeight(hp);
      setBlockBrightness();
      switch ( type ) {
        case 1:
        case 2:
        case 3:
          int deg = (rnd.nextInt()%60);
          if ( deg < 0 ) deg += 360;
          ballManager.addBall(bcx, bcy, deg, 0.2f, -150);
          break;
      }
    }
  }

  private float mv;

  public final void move() {
    if ( mv != 0 ) {
      mv *= -0.9f;
      if ( mv>-0.03f && mv<0.03f ) mv = 0;
      x = cx + rnd.nextFloat()*mv;
      y = cy + rnd.nextFloat()*mv;
    }
    if ( z < 0 ) z += 1f;
  }

  public final boolean hitExists() {
    return (existf && z>=0 );
  }

  public final boolean exists() {
    return existf;
  }
}

