package jp.gr.java_conf.abagames.brick;

import gl4java.*;

/**
  manage blocks
  */
public class BlockManager {
  private static final int WIDTH = 14, HEIGHT = 16, SIZE = WIDTH*HEIGHT;

  private Block[] blocks;
  private StageData stageData;
  private GameManager gameManager;

  public BlockManager(Racket racket, GameManager gameManager, GLFunc gl) {
    this.gameManager = gameManager;
    blocks = new Block[SIZE];
    for ( int i=SIZE-1 ; i>=0 ; i-- ) {
      blocks[i] = new Block(racket, gameManager, gl);
    }
  }

  private BallManager ballManager;

  public void bindBallManager(BallManager ballManager) {
    this.ballManager = ballManager;
    for ( int i=SIZE-1 ; i>=0 ; i-- ) {
      blocks[i].bindBallManager(ballManager);
    }
  }

  public void bindStageData() {
    stageData = new StageData(this);
  }

  public void init() {
    rndClr = true;
    stageData.init();
  }

  public void initDemo() {
    rndClr = true;
  }
  
  public void ballReflected() {
    gameManager.ballReflected();
    stageData.setNextStage();
    rndClr = false;
    gameManager.playSe(2);
  }

  public void setStageData(int stage) {
    stageData.setStage(stage);
  }

  public void clear() {
    for ( int i=SIZE-1 ; i>=0 ; i-- ) {
      blocks[i].clear();
    }
  }

  private int bIdx = SIZE-1;

  public static final float BLKWDT = Wall.WIDTH*2/WIDTH;
  public static final float BLKHGT = BLKWDT/2;
  public static final float YPOS = 2;

  public void addBlock(int x, int y, int type, int hp) {
    float xp = x*BLKWDT-Wall.WIDTH;
    float yp = Wall.HEIGHT-y*BLKHGT-YPOS;
    blocks[x+y*WIDTH].set(xp, yp, type, hp);
  }

  private void xReflect(Ball ball) {
    ball.mx = -ball.mx;
    ball.x = ball.px+ball.mx;
  }

  private void yReflect(Ball ball) {
    ball.my = -ball.my;
    ball.y = ball.py+ball.my;
  }

  public final boolean checkHit(Ball ball) {
    int xIdx = (int)((ball.x+Wall.WIDTH)/BLKWDT);
    int yIdx = (int)((-ball.y+-YPOS+Wall.HEIGHT)/BLKHGT);
    if ( yIdx<0 || yIdx>=HEIGHT ) return false;
    Block block = blocks[xIdx + yIdx*WIDTH];
    if ( block.hitExists() ) {
      block.hit();
      float hby1 = Wall.HEIGHT-yIdx*BLKHGT-YPOS;
      float hby2 = hby1-BLKHGT;
      if ( ball.py<=hby1 && ball.py>=hby2 ) {
        if ( ball.mx > 0 ) {
          xIdx--;
          if ( xIdx >= 0 ) {
            if ( blocks[xIdx + yIdx*WIDTH].exists() ) {
              yReflect(ball);
              return true;
            }
          }
        } else {
          xIdx++;
          if ( xIdx < WIDTH ) {
            if ( blocks[xIdx + yIdx*WIDTH].exists() ) {
              yReflect(ball);
              return true;
            }
          }
        }
        xReflect(ball);
        return true;
      } else {
        if ( ball.my < 0 ) {
          yIdx--;
          if ( yIdx >= 0 ) {
            if ( blocks[xIdx + yIdx*WIDTH].exists() ) {
              xReflect(ball);
              return true;
            }
          }
        } else {
          yIdx++;
          if ( yIdx < HEIGHT ) {
            if ( blocks[xIdx + yIdx*WIDTH].exists() ) {
              xReflect(ball);
              return true;
            }
          }
        }
        yReflect(ball);
        return true;
      }
    }
    return false;
  }

  public boolean rndClr;

  public final void move() {
    boolean ef = false;
    for ( int i=SIZE-1 ; i>=0 ; i-- ) {
      if ( blocks[i].exists() ) {
        blocks[i].move();
        ef = true;
      }
    }
    if ( !rndClr && !ef ) {
      ballManager.roundCleared();
      rndClr = true;
    }
  }

  private final int WDTDV2 = WIDTH/2;

  public final void draw() {
    for ( int y=0 ; y<HEIGHT ; y++ ) {
      for ( int x=0 ; x<WDTDV2 ; x++ ) {
        int i = x + y*WIDTH;
        if ( blocks[i].exists() ) {
          blocks[i].draw();
        }
      }
      for ( int x=WIDTH-1 ; x>=WDTDV2 ; x-- ) {
        int i = x + y*WIDTH;
        if ( blocks[i].exists() ) {
          blocks[i].draw();
        }
      }
    }
  }
}

