package jp.gr.java_conf.abagames.util.gl4j;

import gl4java.*;
import gl4java.awt.*;
import java.awt.Graphics;

/**
  OpenGL Scene Manager
  */
public class Scene extends GLCanvas {
  public GLFunc glFunc;
  private IGameManager gameManager;

  public Scene(int width, int height, IGameManager gameManager) {
    super(width, height);
    glFunc = gl;
    this.gameManager = gameManager;
  }

  public void display() {
    try {
      clear();
      gameManager.display();
      swap();
    } catch ( Exception e ) {
      return;
    }
  }

  public void init() {
		gl.glClearColor( 0f, 0f, 0f, 0f );
		gl.glFrontFace( GL_CW );

		gl.glDisable(GL_DEPTH_TEST);
		gl.glEnable(GL_CULL_FACE);
		gl.glDisable(GL_ALPHA_TEST);
		gl.glDisable(GL_TEXTURE);
    gl.glDisable(GL_DITHER);

    glj.gljCheckGL();

		int width  = cvsGetWidth();
		int height = cvsGetHeight();

		gl.glViewport( 0, 0, width, height );

	  gl.glMatrixMode(GL_PROJECTION);
	  gl.glLoadIdentity();
	  glu.gluPerspective(90.0, width/height, 1.0, 100.0);
	  gl.glMatrixMode(GL_MODELVIEW);
	  gl.glLoadIdentity();

    //gl.glClear(GL_COLOR_BUFFER_BIT);

    gameManager.init();
  }

  private boolean rsf = false;
  private int rsWidth, rsHeight;

  public void resized(int width, int height) {
    rsf = true;
    rsWidth = width; rsHeight = height;
  }

  /**
    Set the viewpoint
    */
  public void setViewpoint(
   double ex, double ey, double ez, double cx, double cy, double cz) {
    glu.gluLookAt(ex, ey, ez, cx, cy, cz, 0, 1.0, 0);
  }

  public void clear() throws SceneStartException {
		if( glj.gljMakeCurrent(true) == false ) {
			throw new SceneStartException();
		}
    gl.glClear(GL_COLOR_BUFFER_BIT);
  }

  public void makeCurrent() throws SceneStartException {
		if( glj.gljMakeCurrent(true) == false ) {
			throw new SceneStartException();
		}
  }

  public void swap() {
    if ( rsf ) {
      rsf = false;
      gl.glViewport (0, 0, rsWidth, rsHeight);
      gl.glLoadIdentity();
    }
    glj.gljSwap();
    glj.gljCheckGL();
    glj.gljFree();
    if ( destroyd ) cvsDispose();
  }

  private boolean destroyd = false;

  public void destroy() {
    destroyd = true;
  }
}

